﻿using Newtonsoft.Json;
using PoE_MxFilterGen.generator;
using PoE_MxFilterGen.json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace PoE_MxFilterGen
{
    internal class main
    {
        private static DateTime dt = DateTime.Now;
        public static string version = "10.5.1";
        public static string fDate = string.Format("{0}-{1}-{2}", (object)main.dt.Day, (object)main.dt.Month, (object)main.dt.Year);
        public static string api = "";
        public static string giturl = "https://mxgit.ovh/mikx";
        public static string section = "";
        public static string league = "";
        public static string path = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
        public static int fprog = 0;
        public static int ftotal = 0;
        public static int sprog = 0;
        public static int stotal = 0;
        public static bool debug = false;

        private static void Main(string[] args)
        {
            if (!File.Exists("config/mxfiltergen_settings.json"))
            {
                msg.CMW("ERROR: settings.json not found! Downloading a template...", true, 3);
                web.DownloadFile("https://mxpoe.ovh/mxfilter/data/settings_template.json", "config/mxfiltergen_settings.json");
            }
            msg.Splash();
            main.CleanDirData();
            main.league = JsonConvert.DeserializeObject<main.REMVAR>(File.ReadAllText("config/mxfiltergen_var.json")).league;
            main.api = "https://poe.ninja/api/data/";
            msg.CMW("League: " + main.league, true, 1);
            msg.CMW(string.Format("Min. Unique Value - {0}c", (object)settings.GetUniqueValue()), true, 1);
            msg.CMW(string.Format("Min. Fossil Value - {0}c", (object)settings.GetFossilValue()), true, 1);
            msg.CMW(string.Format("Min. Card Value   - {0}c", (object)settings.GetCardValue()), true, 1);
            msg.CMW("Checking for required dirs...", true, 1);
            if (!Directory.Exists("data"))
                Directory.CreateDirectory("data");
            if (!Directory.Exists("gen"))
                Directory.CreateDirectory("gen");
            if (!Directory.Exists("structure"))
                Directory.CreateDirectory("structure");
            msg.CMW("Cleaning the base dirs...", true, 1);
            main.CleanDirData();
            msg.CMW("Cleaning the last filter from path...", true, 1);
            foreach (string filter in structures.Filters())
                File.Delete("out/MxFilter_" + filter + ".filter");
            msg.CMW("Downloading the latest API data from poe.ninja...", true, 1);
            web.SaveString(main.api + "itemoverview?league=" + main.league + "&type=UniqueArmour&language=en", "data/ninja.armour.json");
            web.SaveString(main.api + "itemoverview?league=" + main.league + "&type=UniqueWeapon&language=en", "data/ninja.weapon.json");
            web.SaveString(main.api + "itemoverview?league=" + main.league + "&type=UniqueAccessory&language=en", "data/ninja.accessory.json");
            web.SaveString(main.api + "itemoverview?league=" + main.league + "&type=DivinationCard&language=en", "data/ninja.card.json");
            web.SaveString(main.api + "currencyoverview?league=" + main.league + "&type=Currency&language=en", "data/ninja.currency.json");
            web.SaveString(main.api + "itemoverview?league=" + main.league + "&type=Fossil&language=en", "data/ninja.fossil.json");
            web.SaveString(main.api + "itemoverview?league=" + main.league + "&type=BaseType&language=en", "data/ninja.base.json");
            foreach (string filter in structures.Filters())
            {
                string str = "MxFilter";
                int sc = structures.Structures().Length;
                msg.CMW(string.Format("Generating the {0} filter using {1} source(s)...", filter, sc), true, 1);
                main.ftotal = sc;
                foreach (var structure in structures.Structures())
                {
                    ++main.fprog;
                    msg.drawProgress(main.fprog, main.ftotal);
                    settings.WriteSection(structure);
                    string section = structure;
                    int fossilValue = settings.GetFossilValue();
                    int cardValue = settings.GetCardValue();
                    int uniqueValue = settings.GetUniqueValue();
                    int tierValue = settings.GetTierValue();
                    bool tier1HAxe = settings.GetTier1HAxe();
                    bool tier2HAxe = settings.GetTier2HAxe();
                    bool tierBow = settings.GetTierBow();
                    bool tierClaw = settings.GetTierClaw();
                    bool tierDagger = settings.GetTierDagger();
                    bool tier1HMace = settings.GetTier1HMace();
                    bool tier2HMace = settings.GetTier2HMace();
                    bool tierSceptre = settings.GetTierSceptre();
                    bool tierStaff = settings.GetTierStaff();
                    bool tier1HSword = settings.GetTier1HSword();
                    bool tier2HSword = settings.GetTier2HSword();
                    bool tierWand = settings.GetTierWand();
                    bool tierBody = settings.GetTierBody();
                    bool tierBoots = settings.GetTierBoots();
                    bool tierGloves = settings.GetTierGloves();
                    bool tierHelmet = settings.GetTierHelmet();
                    bool tierShield = settings.GetTierShield();
                    bool tierQuiver = settings.GetTierQuiver();
                    switch (structure)
                    {
                        case "DIVINATION CARD (GEN)":
                            card.Gen(section, cardValue);
                            break;
                        case "EXPENSIVE UNIQUE (GEN) (Accessories)":
                            accessory.Gen(section, uniqueValue);
                            break;
                        case "EXPENSIVE UNIQUE (GEN) (Armours)":
                            armour.Gen(section, uniqueValue);
                            break;
                        case "EXPENSIVE UNIQUE (GEN) (Weapons)":
                            weapon.Gen(section, uniqueValue);
                            break;
                        case "FOSSIL (GEN)":
                            fossil.Gen(section, fossilValue);
                            break;
                        case "Tiers (DATA)":
                            tier.Gen(section, tierValue, tier1HAxe, tier2HAxe, tierBow, tierClaw, tierDagger, tier1HMace, tier2HMace, tierSceptre, tierStaff, tier1HSword, tier2HSword, tierWand, tierBody, tierBoots, tierGloves, tierHelmet, tierShield, tierQuiver);
                            break;
                    }
                }
                msg.CMW("Creating the final filter...", true, 1);
                foreach (var structure in structures.Structures())
                {
                    switch (structure)
                    {
                        case "DIVINATION CARD (GEN)":
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", File.ReadAllText(string.Format("gen/" + structure + ".filter")));
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            break;
                        case "EXPENSIVE UNIQUE (GEN) (Accessories)":
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", File.ReadAllText(string.Format("gen/" + structure + ".filter")));
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            break;
                        case "EXPENSIVE UNIQUE (GEN) (Armours)":
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", File.ReadAllText(string.Format("gen/" + structure + ".filter")));
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            break;
                        case "EXPENSIVE UNIQUE (GEN) (Weapons)":
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", File.ReadAllText(string.Format("gen/" + structure + ".filter")));
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            break;
                        case "FOSSIL (GEN)":
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", File.ReadAllText(string.Format("gen/" + structure + ".filter")));
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            break;
                        case "Tiers (DATA)":
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", File.ReadAllText(string.Format("gen/" + structure + ".filter")));
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            break;
                        case "Tiers - Caster (DATA)":
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", File.ReadAllText(string.Format("gen/" + structure + ".filter")));
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            break;
                            default:
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", string.Format("# Section: {0}", (object)structure) + Environment.NewLine);
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", File.ReadAllText(string.Format("structure/" + filter + "/" + structure + ".filter")));
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            File.AppendAllText("out/" + str + "_" + filter + ".filter", Environment.NewLine ?? "");
                            break;
                    }
                }
                main.ftotal = 0;
                main.fprog = 0;
                main.CleanDir();
                CurrencyValueOut();
                if(debug)
                {
                    Console.ReadKey();
                }               
            }
        }

        public static void CurrencyValueOut()
        {
            foreach (LineCurrency line in JsonConvert.DeserializeObject<RootCurrency>(File.ReadAllText("data/ninja.currency.json", Encoding.UTF8)).lines)
            {
                string currencyTypeName = line.currencyTypeName;

                if (currencyTypeName == "Divine Orb")
                {
                    string fc = line.chaosEquivalent.ToString();
                    File.WriteAllText("out/divine.txt", fc);
                }

                if (currencyTypeName == "Exalted Orb")
                {
                    string fc = line.chaosEquivalent.ToString();
                    File.WriteAllText("out/exalted.txt", fc);
                }

                if (currencyTypeName == "Orb of Annulment")
                {
                    string fc = line.chaosEquivalent.ToString();
                    File.WriteAllText("out/annulment.txt", fc);
                }

                if (currencyTypeName == "Orb of Alteration")
                {
                    string fc = line.chaosEquivalent.ToString();
                    File.WriteAllText("out/alteration.txt", fc);
                }

                if (currencyTypeName == "Orb of Fusing")
                {
                    string fc = line.chaosEquivalent.ToString();
                    File.WriteAllText("out/fusing.txt", fc);
                }

                if (currencyTypeName == "Chromatic Orb")
                {
                    string fc = line.chaosEquivalent.ToString();
                    File.WriteAllText("out/chromatic.txt", fc);
                }

                if (currencyTypeName == "Jeweller's Orb")
                {
                    string fc = line.chaosEquivalent.ToString();
                    File.WriteAllText("out/jeweller.txt", fc);
                }

                if (currencyTypeName == "Orb of Alchemy")
                {
                    string fc = line.chaosEquivalent.ToString();
                    File.WriteAllText("out/alchemy.txt", fc);
                }

                if (currencyTypeName == "Cartographer's Chisel")
                {
                    string fc = line.chaosEquivalent.ToString();
                    File.WriteAllText("out/chisel.txt", fc);
                }
            }
        }

        public static void CleanDirData()
        {
            DirectoryInfo directoryInfo1 = new DirectoryInfo("data");
            DirectoryInfo directoryInfo2 = new DirectoryInfo("gen");
            foreach (FileSystemInfo file in directoryInfo1.GetFiles())
                file.Delete();
            foreach (FileSystemInfo file in directoryInfo2.GetFiles())
                file.Delete();
        }

        public static void CleanDir()
        {
            foreach (FileSystemInfo file in new DirectoryInfo("gen").GetFiles())
                file.Delete();
        }

        private static void AutoUpdater_ApplicationExitEvent() => Environment.Exit(-1);

        public class REMVAR
        {
            public string league { get; set; }
        }

        public class REMSND
        {
            public List<string> sound { get; set; }
        }

        public class STRUCTURE
        {
            public string section { get; set; }

            public bool gen { get; set; }

            public bool @static { get; set; }

            public bool remote { get; set; }

            public string url { get; set; }
        }

        public class RootStructure
        {
            public List<main.STRUCTURE> structures { get; set; }
        }
    }
}